EnableExplicit

;Conway's Game of Life
;siehe https://de.wikipedia.org/wiki/Conways_Spiel_des_Lebens
;oder https://en.wikipedia.org/wiki/Conway%27s_Game_of_Life
;siehe auch http://www.conwaylife.com/wiki/Main_Page

;Die Pattern knnen hier heruntergeladen werden : http://www.conwaylife.com/wiki/Main_Page
;die Pattern stehen im Projektverzeichnis in "ConwayPatterns\"

; Since Life's inception, new similar cellular automata have been developed. 
; The standard Game of Life is symbolized As "B3/S23": 
; A cell is "Born" If it has exactly 3 neighbours, "Survives" If it has 2 Or 3 living neighbours; 
; it dies otherwise. The first number, Or List of numbers, is what is required For a dead cell To be born. 
; The second set is the requirement For a live cell To survive To the Next generation. 
; Hence "B6/S16" means "a cell is born if there are 6 neighbours, 
; And lives on If there are either 1 Or 6 neighbours". 


;-Enum
Enumeration   ;ConwayStatus
  #ConwayHaelt
  #ConwayLaeuft  
EndEnumeration

Enumeration    ;Auswahlstatus 
  #AuswahlInaktiv
  #AuswahlWirdAufgezogen
  #AuswahlAktiv 
EndEnumeration

Enumeration PopupMenus 1
  ;Menu im Canvas-Gadget
  #Ausschneiden
  #Kopieren
  #Einfuegen
  #SpiegelnHorizontal
  #SpiegelnVertikal
  #Drehen90Rechts
  #Drehen90Links
  #Drehen180
  #Sichern
  #ObjektUmbenennnen
  #ObjektLoeschen
  #ObjektListeLoeschen
  #ZufaelligFuellen
  #Zoomen
  ;Menu in ListIcon-Gadget "Geladene Objekte" bzw. "Eigene Objekte"
  #Objektinfo
  #ObjektEntfernen
  
  ;Menu "Objekt importieren"
  #ImportAusZwischenablage
  #ImportAusDatei
  
EndEnumeration

;-Const
;Die Farben der Zellen
#ZelleLebt=0
#ZelleTot=$ffffff
#FarbeAuswahlrahmen=$ff00ff

;-Structure
Structure sRegel
  name.s
  List WirdGeboren.i()
  List Ueberlebt.i()
EndStructure

Structure sKoordinaten
  x.w
  y.w
EndStructure

Structure sAuswahl  
  Status.i
  ObenLinks.sKoordinaten
  UntenRechts.sKoordinaten
EndStructure

Structure sObjekte
  Regel.s
  Name.s
  Breite.i
  Hoehe.i
  Infotext.s
  Autor.s
  List RLE_String.s()
EndStructure

Structure sEigeneObjekte Extends sObjekte
  gespeichert.i ;wurde das Objekt auf Disc gespeichert ?
EndStructure

Structure sImport   ;fr ObjektImportieren
  Zoom.i
  XAbstand.i
  XOffset.i
  YAbstand.i
  YOffset.i 
EndStructure

;-Global
Global Zellengroesse=10
Global Conway_Breite=100
Global Conway_Hoehe=100
Global Torus=1
Global Zeit ;so schnell luft das Spiel (in ms)
Global ConwayStatus=#ConwayHaelt
Global Auswahl.sAuswahl
Global lmt,rmt  ;ist 1, wenn jewilige Maustaste gedrckt ist
Global AusgewaehltesObjekt
Global Generation,Population,Population_Minimum,Population_Maximum,Stoppen,Einfuegen
Global ObjektImport.sImport

;Standardregel
Global Regel.sRegel
Regel\name="B3/S23"
AddElement(Regel\WirdGeboren())
Regel\WirdGeboren()=3
AddElement(Regel\Ueberlebt())
Regel\Ueberlebt()=2
AddElement(Regel\Ueberlebt())
Regel\Ueberlebt()=3

Global Dim Conway.b(Conway_Breite+1,Conway_Hoehe+1)
Global Dim Conway_NeueGeneration.b(Conway_Breite+1,Conway_Hoehe+1)
Global Dim AusschneidenKopierenEinfuegen.b(0,0)
Global Dim RLESpeicher.b(0,0)

Global NewList rle.s()
Global NewList Objekte.sObjekte()
Global NewList EigeneObjekte.sEigeneObjekte()

;-Use
UseGIFImageDecoder()

;-Define
Define Event

;-Declare
Declare Zuruecksetzen(et)
Declare ConwayAnzeigen()

;-Include
IncludeFile "GUI.pbf"
IncludeFile "Eingabemaske.pbf"
IncludeFile "ImportierenFenster.pbf"
IncludeFile "RLE_Pattern.pbi"
IncludeFile "Menus bearbeiten.pbi"
IncludeFile "ObjektImportieren.pbi"


;-Macro
Macro  ConwayZelleTesten ;ist etwas schneller als "Procedure ConwayZelleTesten()"
  If c=0                 ;untersuchte Zelle lebt nicht
    ForEach Regel\WirdGeboren()
      If n=Regel\WirdGeboren():b=1:Population+1:Break:EndIf  ;Zelle wird geboren
    Next
    Conway_NeueGeneration(x,y)=b
  ElseIf c=1  ;untersuchte Zelle lebt
    ForEach Regel\Ueberlebt()
      If n=Regel\Ueberlebt():s=1:Population+1:Break:EndIf ;Zelle berlebt
    Next
    Conway_NeueGeneration(x,y)=s
  EndIf    
EndMacro

Procedure ConwayAnzeigen()  
  Protected x,y,x1,y1,sx,sy,ex,ey
  sx=GetGadgetAttribute(#scrollarea_0,#PB_ScrollArea_X)/Zellengroesse ;sichtbaren
  sy=GetGadgetAttribute(#scrollarea_0,#PB_ScrollArea_Y)/Zellengroesse ;Bereich
  ex=sx+800/Zellengroesse                                             ;anzeigen
  ey=sy+800/Zellengroesse
  If ex>Conway_Breite:ex=Conway_Breite:EndIf
  If ey>Conway_Hoehe:ey=Conway_Hoehe:EndIf
  If StartDrawing(CanvasOutput(#Canvas))   
    ;     For y=1 To Conway_Hoehe
    ;       For x=1 To Conway_Breite
    For y=sy To ey
      For x=sx To ex
        If Conway(x,y)
          Box((x-1)*Zellengroesse,(y-1)*Zellengroesse,Zellengroesse,Zellengroesse,#ZelleLebt)
        Else
          Box((x-1)*Zellengroesse,(y-1)*Zellengroesse,Zellengroesse,Zellengroesse,#ZelleTot)
        EndIf
      Next
    Next
    If Auswahl\Status=#AuswahlWirdAufgezogen Or Auswahl\Status=#AuswahlAktiv
      x=(Auswahl\ObenLinks\x-1)*Zellengroesse
      y=(Auswahl\ObenLinks\y-1)*Zellengroesse
      x1=(Auswahl\UntenRechts\x-1)*Zellengroesse
      y1=(Auswahl\UntenRechts\y-1)*Zellengroesse
      DrawingMode(#PB_2DDrawing_Outlined)
      Box(x,y,x1-x,y1-y,#FarbeAuswahlrahmen)
    EndIf  
    StopDrawing()
  EndIf
  If population<population_minimum:population_minimum=population:EndIf
  If population>population_maximum:population_maximum=population:EndIf  
  StatusBarText(1,1,"Generation : "+Str(generation))
  StatusBarText(1,2,"Population : "+Str(population))
  StatusBarText(1,3,"Grte Population : "+Str(population_maximum))
  StatusBarText(1,4,"Kleinste Population : "+Str(population_minimum))
EndProcedure

Procedure ConwayBerechnen()
  Protected x,y,c,n,x1,x2,y1,y2,b,s
  If population=0
    RemoveWindowTimer(#Window_0,1)
    MessageRequester("Game of Life ist beendet","Die Population ist nach "+Str(generation)+" Generationen ausgestorben.",#PB_MessageRequester_Info)
    Zuruecksetzen(0)
    ProcedureReturn 
  EndIf
  generation+1
  If Generation=Stoppen
    RemoveWindowTimer(#Window_0,1)
    ConwayStatus=#ConwayHaelt
    SetGadgetText(#Start0,"WEITER")   
    MessageRequester("Game of Life wurde angehalten","Generation "+Str(generation)+" ist erreicht",#PB_MessageRequester_Info)
    ProcedureReturn 
  EndIf  
  population=0
  For y=1 To Conway_Hoehe
    For x=1 To Conway_Breite
      c=Conway(x,y)
      n=0:b=0:s=0
      ;Wieviele Nachbarzellen "leben" ?
      n=Conway(x-1,y-1)+Conway(x,y-1)+Conway(x+1,y-1)+
        Conway(x-1,y)+Conway(x+1,y)+
        Conway(x-1,y+1)+Conway(x,y+1)+Conway(x+1,y+1)
      ConwayZelleTesten
    Next
  Next
  If Torus=1  ;Rnder berechnen
    For y=1 To Conway_Hoehe 
      x=Conway_Breite:x1=Conway_Breite-1:x2=1 ;rechter Rand von oben nach unten    
      c=Conway(x,y):b=0:s=0
      n=Conway(x1,y-1)+Conway(x,y-1)+Conway(x2,y-1)+
        Conway(x1,y)+Conway(x2,y)+
        Conway(x1,y+1)+Conway(x,y+1)+Conway(x2,y+1)
      ConwayZelleTesten
      x=1:x1=Conway_Breite:x2=2 ;linker Rand von oben nach unten
      c=Conway(x,y):b=0:s=0
      n=Conway(x1,y-1)+Conway(x,y-1)+Conway(x2,y-1)+
        Conway(x1,y)+Conway(x2,y)+
        Conway(x1,y+1)+Conway(x,y+1)+Conway(x2,y+1)
      ConwayZelleTesten
    Next
    For x=1 To Conway_Breite 
      y=Conway_Hoehe:y1=Conway_Hoehe-1:y2=1 ;unterer Rand von links nach rechts    
      c=Conway(x,y):b=0:s=0
      n=Conway(x-1,y1)+Conway(x-1,y)+Conway(x-1,y2)+
        Conway(x,y1)+Conway(x,y2)+
        Conway(x+1,y1)+Conway(x+1,y)+Conway(x+1,y2)
      ConwayZelleTesten
      y=1:y1=Conway_Hoehe:y2=2  ;oberer Rand von links nach rechts
      c=Conway(x,y):b=0:s=0
      n=Conway(x-1,y1)+Conway(x-1,y)+Conway(x-1,y2)+
        Conway(x,y1)+Conway(x,y2)+
        Conway(x+1,y1)+Conway(x+1,y)+Conway(x+1,y2)
      ConwayZelleTesten
    Next    
  EndIf 
  CopyArray(Conway_NeueGeneration(),Conway())   
EndProcedure

Procedure ConwayZyklen(et)
  Select GetGadgetState(#Zyklen)
    Case 0:zeit=1000
    Case 1:zeit=1000/2
    Case 2:zeit=1000/5
    Case 3:zeit=1000/10
    Case 4:zeit=1000/50
    Case 5:zeit=1000/100
  EndSelect
  If ConwayStatus=#ConwayLaeuft
    RemoveWindowTimer(#Window_0,1)
    AddWindowTimer(#Window_0,1,Zeit)
  EndIf
EndProcedure

Procedure ConwayGroesseSetzen(et)
  Protected x=Val(GetGadgetText(#Breite))
  Protected y=Val(GetGadgetText(#Hoehe))
  Zuruecksetzen(0)
  Conway_Breite=x
  Conway_Hoehe=y
  Dim Conway.b(Conway_Breite+1,Conway_Hoehe+1)
  Dim Conway_NeueGeneration.b(Conway_Breite+1,Conway_Hoehe+1)
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_InnerWidth,Zellengroesse*Conway_Breite)
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_InnerHeight,Zellengroesse*Conway_Hoehe)
  ;SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_ScrollStep,400) ; <- fehlt im Formdesigner
  ResizeGadget(#Canvas,#PB_Ignore,#PB_Ignore,Zellengroesse*Conway_Breite,Zellengroesse*Conway_Hoehe) 
  ConwayAnzeigen()
EndProcedure

; Procedure BindScrollArea()
;   Debug GetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_Y)
; EndProcedure

Procedure CanvasRMTgedrueckt(mx,my)
  rmt=1
  If Auswahl\Status=#AuswahlAktiv
    DisplayPopupMenu(1,WindowID(#window_0))
  Else
    If ArraySize(AusschneidenKopierenEinfuegen(),1)>0 And ArraySize(AusschneidenKopierenEinfuegen(),2)>0
      DisplayPopupMenu(2,WindowID(#window_0))
    EndIf  
  EndIf
EndProcedure

Procedure CanvasRMTlosgelassen(mx,my)
  rmt=0
EndProcedure

Procedure CanvasLMTgedrueckt(mx,my)
  lmt=1  
  
EndProcedure

Procedure CanvasLMTlosgelassen(mx,my)
  Protected x,y,c,xz,yz
  lmt=0
  If Einfuegen=1
    For y=0 To ArraySize(AusschneidenKopierenEinfuegen(),2)
      yz=y+my
      If yz>Conway_Hoehe
        If Torus=1:yz-Conway_Hoehe:Else:Break:EndIf
      EndIf
      For x=0 To ArraySize(AusschneidenKopierenEinfuegen(),1)
        xz=x+mx
        If xz>Conway_Breite
          If Torus=1:xz-Conway_Breite:Else:Break:EndIf
        EndIf
        c=AusschneidenKopierenEinfuegen(x,y)
        Conway(xz,yz)=c
        If c:Population+1:EndIf 
      Next
    Next
    ConwayAnzeigen()  
    Einfuegen=0
    SetGadgetAttribute(#canvas,#PB_Canvas_Cursor,#PB_Cursor_Default)
    ProcedureReturn 
  EndIf   
  If Auswahl\Status=#AuswahlInaktiv
    If AusgewaehltesObjekt=0
      ;Conway(mx,my)=1-Conway(mx,my)
      If Conway(mx,my)=0
        Conway(mx,my)=1
        Population+1
      ElseIf Conway(mx,my)=1
        Conway(mx,my)=0
        Population-1
      EndIf
      ConwayAnzeigen()
    ElseIf AusgewaehltesObjekt>0  ;geladenes Objekt ausgewhlt      
      SelectElement(Objekte(),AusgewaehltesObjekt-1)
      RLE_Dekodieren(mx,my,Objekte()\RLE_String())
      AusgewaehltesObjekt=0
      ConwayAnzeigen()
      SetGadgetAttribute(#canvas,#PB_Canvas_Cursor,#PB_Cursor_Default)
    Else ;Eigenes Objekt ausgewhlt
      AusgewaehltesObjekt*-1
      SelectElement(EigeneObjekte(),AusgewaehltesObjekt-1)
      RLE_Dekodieren(mx,my,EigeneObjekte()\RLE_String())
      AusgewaehltesObjekt=0
      ConwayAnzeigen()
      SetGadgetAttribute(#canvas,#PB_Canvas_Cursor,#PB_Cursor_Default)            
    EndIf
  ElseIf Auswahl\Status=#AuswahlWirdAufgezogen   
    Auswahl\Status=#AuswahlAktiv
    DisplayPopupMenu(1,WindowID( #window_0))    
  ElseIf Auswahl\Status=#AuswahlAktiv
    Auswahl\Status=#AuswahlInaktiv
    Auswahl\ObenLinks\x=-10000:Auswahl\ObenLinks\y=-10000
    ConwayAnzeigen()
  EndIf 
EndProcedure

Procedure CanvasMausBewegt(mx,my)
  Protected x,y,c,zx,zy,w,b,z,sx,sy
  If GetGadgetState(#Option_0)=1:w=0:EndIf
  If GetGadgetState(#Option_1)=1:w=30:EndIf
  If GetGadgetState(#Option_2)=1:w=60:EndIf
  ;Debug mx
  If lmt=1
    If Auswahl\Status=#AuswahlInaktiv
      If Abs(Auswahl\ObenLinks\x-mx)>30 And Abs(Auswahl\ObenLinks\y-my)>30
        Auswahl\Status=#AuswahlWirdAufgezogen
        Auswahl\ObenLinks\x=mx
        Auswahl\ObenLinks\y=my
      EndIf
    ElseIf Auswahl\Status=#AuswahlWirdAufgezogen
      Auswahl\UntenRechts\x=mx
      Auswahl\UntenRechts\y=my
      ConwayAnzeigen()
    EndIf
  EndIf
  If w                          ;ZoomCanvas zeichnen
    z=300/w
    b=z/2
    sx=mx-b
    sy=my-b
    If StartDrawing(CanvasOutput(#Zoomgadget))
      For y=0 To z
        For x=0 To z
          zx=sx+x
          zy=sy+y
          If zx<0:zx=0:EndIf
          If zy<0:zy=0:EndIf
          If zx>Conway_Breite:zx=Conway_Breite:EndIf
          If zy>Conway_Hoehe:zy=Conway_Hoehe:EndIf 
          c=Conway(zx,zy)
          DrawingMode(#PB_2DDrawing_Default)
          If c=0
            Box(x*w,y*w,w,w,#ZelleTot)
          Else
            Box(x*w,y*w,w,w,#ZelleLebt)
          EndIf
          If x=b And y=b
            DrawingMode(#PB_2DDrawing_AlphaBlend)
            Box(x*w,y*w,w,w,$80888888)
            ;Circle(x*w+w/2,y*w+w/2,w/2,$888888)
          EndIf
        Next
      Next   
      StopDrawing()
    EndIf
  EndIf  
  StatusBarText(1,0,"Mauskoordinaten : "+Str(mx)+"/"+Str(my))  
EndProcedure

Procedure BindCanvasGadget()                               ;<--- Maus ist im Canvasgadget --->
  Protected et=EventType()
  Protected mx=GetGadgetAttribute(#Canvas,#PB_Canvas_MouseX)/Zellengroesse+1
  Protected my=GetGadgetAttribute(#Canvas,#PB_Canvas_MouseY)/Zellengroesse+1
  If ConwayStatus=#ConwayHaelt
    Select et
      Case #PB_EventType_LeftButtonDown
        CanvasLMTgedrueckt(mx,my)
      Case #PB_EventType_MouseMove
        CanvasMausBewegt(mx,my)
      Case #PB_EventType_LeftButtonUp
        CanvasLMTlosgelassen(mx,my)
      Case #PB_EventType_RightButtonDown
        CanvasRMTgedrueckt(mx,my)
      Case #PB_EventType_RightButtonUp
        CanvasRMTlosgelassen(mx,my)
      Case #PB_EventType_MouseLeave
        If StartDrawing(CanvasOutput(#Zoomgadget))
          Box(0,0,300,300,#ZelleTot)
          StopDrawing()
        EndIf
        
    EndSelect
  EndIf
  
EndProcedure

Procedure Window(event,win)
  
EndProcedure

Procedure Start(et)         ;Startgadget angeklickt 
  If ConwayStatus=#ConwayHaelt
    AddWindowTimer(#Window_0,1,Zeit)
    ConwayStatus=#ConwayLaeuft
    SetGadgetText(#Start0,"STOP")
    population_maximum=population
    population_minimum=population
    Stoppen=Val(GetGadgetText(#Stoppen))
  ElseIf ConwayStatus=#ConwayLaeuft
    RemoveWindowTimer(#Window_0,1)
    ConwayStatus=#ConwayHaelt
    SetGadgetText(#Start0,"WEITER")    
  EndIf
EndProcedure

Procedure Einzelschritt(et)
  ;Protected zeit=ElapsedMilliseconds()
  ConwayBerechnen()
  ConwayAnzeigen()
  If ConwayStatus=#ConwayLaeuft
    RemoveWindowTimer(#Window_0,1)
    ConwayStatus=#ConwayHaelt
  EndIf
  SetGadgetText(#Start0,"WEITER")      
  ;Debug "Zeit fr Conway_Berechnen:"+Str(ElapsedMilliseconds()-zeit)
EndProcedure

Procedure Zuruecksetzen(et)
  Protected x,y
  ConwayStatus=#ConwayHaelt
  RemoveWindowTimer(#Window_0,1)
  SetGadgetText(#Start0,"START")      
  For y=0 To Conway_Hoehe+1
    For x=0 To Conway_Breite+1
      Conway(x,y)=0
    Next
  Next
  CopyArray(Conway(),Conway_NeueGeneration())
  Generation=0
  Population=0
  ConwayAnzeigen()
EndProcedure

Procedure Torus(et)
  Torus=GetGadgetState(#Checkbox_0)
EndProcedure

Procedure Laden(et)
  ;Protected name.s=OpenFileRequester("RLE-Pattern laden...",GetCurrentDirectory()+"ConwayPatterns\","RLE-Dateien|*.rle",0) 
  Protected name.s=OpenFileRequester("RLE-Pattern laden... "+" Nach dem Laden mit LMT in Spielfeld platzieren","","RLE-Dateien|*.rle",0) 
  If name
    RLE_DateiLaden(name)
    ConwayAnzeigen()
  EndIf
EndProcedure

Procedure Zoom(et)
  Protected z,b,h
  Select GetGadgetState(#Zoom) 
    Case 0:z=0
    Case 1:z=1
    Case 2:z=2
    Case 3:z=5
    Case 4:z=10
    Case 5:z=20
  EndSelect  
  If z=0  ;Passend zoomen   
    b=800/Conway_Breite
    h=800/Conway_Hoehe
    If b<h
      z=b
    Else
      z=h
    EndIf
    If z<1:z=1:EndIf
    If z>25:z=25:EndIf 
  EndIf
  Zellengroesse=z
  ;If (x>GadgetWidth(#Canvas)) Or (y>GadgetHeight(#Canvas))
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_InnerWidth,Zellengroesse*Conway_Breite)
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_InnerHeight,Zellengroesse*Conway_Hoehe)
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_ScrollStep,400) ; <- fehlt im Formdesigner
  ResizeGadget(#Canvas,#PB_Ignore,#PB_Ignore,Zellengroesse*Conway_Breite,Zellengroesse*Conway_Hoehe) 
  ;EndIf
  ConwayAnzeigen()  
EndProcedure

Procedure GeladeneObjekteliste(et)
  If et=#PB_EventType_LeftClick    
    AusgewaehltesObjekt=GetGadgetState(#GeladeneObjekte)+1
    SetGadgetAttribute(#canvas,#PB_Canvas_Cursor,#PB_Cursor_Cross)
    SelectElement(Objekte(),AusgewaehltesObjekt-1)
    ClearList(Regel\Ueberlebt())
    ClearList(Regel\WirdGeboren())
    Regel\name=""   
    RLE_Regel(Objekte()\Regel)
  EndIf
  If et=#PB_EventType_RightClick    
    AusgewaehltesObjekt=GetGadgetState(#GeladeneObjekte)+1
    If AusgewaehltesObjekt>0
      DisplayPopupMenu(3,WindowID(#window_0))
    EndIf  
  EndIf  
EndProcedure

Procedure EigeneObjekteliste(et)
  If et=#PB_EventType_LeftClick    
    AusgewaehltesObjekt=GetGadgetState(#EigeneObjekte)+1    
    SetGadgetAttribute(#canvas,#PB_Canvas_Cursor,#PB_Cursor_Cross)
    SelectElement(EigeneObjekte(),AusgewaehltesObjekt-1)
    AusgewaehltesObjekt*-1
    ClearList(Regel\Ueberlebt())
    ClearList(Regel\WirdGeboren())
    Regel\name=""   
    RLE_Regel(EigeneObjekte()\Regel)    
  EndIf  
  If et=#PB_EventType_RightClick    
    AusgewaehltesObjekt=GetGadgetState(#EigeneObjekte)+1
    AusgewaehltesObjekt*-1
    If AusgewaehltesObjekt<0
      DisplayPopupMenu(4,WindowID(#window_0))
    EndIf
  EndIf    
EndProcedure

Procedure Importieren(et)
  DisplayPopupMenu(5,WindowID(#Window_0))
  
EndProcedure

Procedure InitGUI()
  Protected t.s
  OpenWindow_0()
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_InnerWidth,Zellengroesse*Conway_Breite)
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_InnerHeight,Zellengroesse*Conway_Hoehe)
  SetGadgetAttribute(#ScrollArea_0,#PB_ScrollArea_ScrollStep,400) ; <- fehlt im Formdesigner
  ResizeGadget(#Canvas,#PB_Ignore,#PB_Ignore,Zellengroesse*Conway_Breite,Zellengroesse*Conway_Hoehe)
  AddGadgetItem(#Zyklen,-1,"  1 Zyklus / Sekunde")
  AddGadgetItem(#Zyklen,-1,"  2 Zyklen / Sekunde")
  AddGadgetItem(#Zyklen,-1,"  5 Zyklen / Sekunde")
  AddGadgetItem(#Zyklen,-1," 10 Zyklen / Sekunde")
  AddGadgetItem(#Zyklen,-1," 50 Zyklen / Sekunde")
  AddGadgetItem(#Zyklen,-1,"100 Zyklen / Sekunde")
  SetGadgetState(#Zyklen,3)
  SetGadgetText(#Breite,Str(Conway_Breite))
  SetGadgetText(#Hoehe,Str(Conway_Hoehe))
  AddKeyboardShortcut(0,#PB_Shortcut_Return,150)
  AddGadgetItem(#Zoom,-1,"Passend zoomen")
  AddGadgetItem(#Zoom,-1,"Zoom 1:1")
  AddGadgetItem(#Zoom,-1,"Zoom 2:1")
  AddGadgetItem(#Zoom,-1,"Zoom 5:1")
  AddGadgetItem(#Zoom,-1,"Zoom 10:1")
  AddGadgetItem(#Zoom,-1,"Zoom 20:1")
  SetGadgetState(#Zoom,4)
  ConwayZyklen(0)
  ;BindGadgetEvent(#ScrollArea_0,@BindScrollArea())
  BindGadgetEvent(#Canvas,@BindCanvasGadget()) 
  If CreateStatusBar(1,WindowID(#Window_0))
    AddStatusBarField(#PB_Ignore)
    AddStatusBarField(#PB_Ignore)
    AddStatusBarField(#PB_Ignore)
    AddStatusBarField(#PB_Ignore)
    AddStatusBarField(#PB_Ignore)
  EndIf
  ForEach Regel\WirdGeboren()
    t+Str(Regel\WirdGeboren())
  Next
  SetGadgetText( #WirdGeboren,t)
  t=""
  ForEach Regel\Ueberlebt()
    t+Str(Regel\Ueberlebt())
  Next
  SetGadgetText( #Ueberlebt,t)  
  SetGadgetState(#Option_1,1)
  If CreatePopupMenu(1)
    MenuItem(#Ausschneiden,"Ausschneiden")
    MenuItem(#Kopieren,"Kopieren")
    MenuItem(#Einfuegen,"Einfgen")
    MenuBar()
    MenuItem(#Zoomen,"Auf Auswahl zoomen")
    MenuBar()
    MenuItem(#SpiegelnHorizontal,"Horizontal spiegeln")
    MenuItem(#SpiegelnVertikal,"Vertikal spiegeln")
    MenuItem(#Drehen90Rechts,"Drehen 90 nach rechts")
    MenuItem(#Drehen90Links,"Drehen 90 nach links")
    MenuItem(#Drehen180,"Drehen 180")
    MenuBar()
    MenuItem(#ZufaelligFuellen,"Zufllig fllen")
    MenuBar()
    MenuItem(#Sichern,"Als eigenes Objekt sichern")
  EndIf
  If CreatePopupMenu(2)
    MenuItem(#Einfuegen,"Einfgen")
  EndIf
  If CreatePopupMenu(3)   ;Menu "Geladene Objekte"
    MenuItem(#Objektinfo,"Info")
    MenuItem(#ObjektEntfernen,"Objekt aus Liste entfernen")
    
  EndIf
  If CreatePopupMenu(4)   ;Menu "Eigene Objekte"
    MenuItem(#Objektinfo,"Info")
    MenuItem(#ObjektEntfernen,"Objekt aus Liste entfernen")
    
  EndIf  
  If CreatePopupMenu(5)  ;Menu "Objekt importieren"
    MenuItem(#ImportAusDatei,"Aus Datei importieren")
    MenuItem(#ImportAusZwischenablage,"Aus Zwischenablage importieren")
  EndIf
  
EndProcedure

Procedure Auswahlmenu(m)
  Select m
      ;Menu im Canvas-Gadget  
    Case #ZufaelligFuellen:AuswahlZufaelligFuellen()
    Case #Kopieren:AuswahlKopieren()
    Case #Einfuegen   
      Einfuegen=1
      SetGadgetAttribute(#canvas,#PB_Canvas_Cursor,#PB_Cursor_Cross)
    Case #Ausschneiden:AuswahlAusschneiden()
    Case #SpiegelnHorizontal:AuswahlSpiegeln(#SpiegelnHorizontal)
    Case #SpiegelnVertikal:AuswahlSpiegeln(#SpiegelnVertikal)
    Case #Drehen180:AuswahlDrehen(#Drehen180)
    Case #Drehen90Links:AuswahlDrehen(#Drehen90Links)
    Case #Drehen90Rechts:AuswahlDrehen(#Drehen90Rechts)
    Case #Zoomen:AuswahlZoomen()
    Case #Sichern:AuswahlSichern()
      ;Menu in ListIcon-Gadget "Geladene Objekte" bzw. igene Objekte"    
    Case #Objektinfo:ObjektInfo()
    Case #ObjektEntfernen:ObjektAusListeEntfernen()
      
      ;Menu "Objekt importieren
    Case  #ImportAusDatei:ObjektAusDateiImportieren()
    Case #ImportAusZwischenablage:ObjektAusZwischenablageImportieren()
      
  EndSelect
EndProcedure

Procedure RegelGadget(et)
  Protected c
  Protected b.s=GetGadgetText(#WirdGeboren),s.s=GetGadgetText(#Ueberlebt)
  If et=#PB_EventType_Change
    ClearList(Regel\Ueberlebt())
    ClearList(Regel\WirdGeboren())
    Regel\name="b"+b+"/s"+s
    For c=1 To Len(b)
      AddElement(Regel\WirdGeboren())
      Regel\WirdGeboren()=Val(Mid(b,c,1))
    Next
    For c=1 To Len(s)
      AddElement(Regel\Ueberlebt())
      Regel\Ueberlebt()=Val(Mid(s,c,1))
    Next
  EndIf  
EndProcedure

Procedure Programmende(et)
  End
EndProcedure

;-Main
InitGUI()

;Zum Testen der Regeleingaben laden (Regel ist 125/36)
;RLE_DateiLaden("ConwayPatterns\2x2glider.rle")

;RLE_DateiLaden("ConwayPatterns\101.rle")
;ConwayAnzeigen()

;Zum testen der Procedure "ObjektAusDateiImportieren()"  
; LoadImage(1,"E:\PureBasic\Conways Game Of Life\G3_unruhe.gif")
; SetClipboardImage(1)
; FreeImage(1)
; ObjektAusZwischenablageImportieren()
; ObjektAusDateiImportieren()


;-MainLoop
Repeat
  Event=WaitWindowEvent()
  
  If Event=#PB_Event_Gadget   ;Auswahlrahmen lschen
    If EventGadget()<>#canvas
      Auswahl\Status=#AuswahlInaktiv
      ConwayAnzeigen()
    EndIf
  EndIf  
  If Event=#PB_Event_Timer
    ConwayBerechnen()
    ConwayAnzeigen()
  EndIf
  If event=#PB_Event_Menu
    Auswahlmenu(EventMenu())
  EndIf 
  
Until Window_0_Events(Event)=#False

;TODO Bereich um Mauszeiger im Zoomgadget (das Canvas unten rechts) anzeigen -> OK
;TODO Objekt aus Grafik importieren (Zwischenablage ober Datei)
;TODO Regel in EigeneObjekte eintragen und anwenden ! -> testen
;TODO Procedure "RegelGadget()" testen
;TODO Nur den sichtbaren Bereich zeichnen, ist schneller als alles zu zeichnen -> testen
;TODO Stopbedingung - Anhalten wenn Population oder Generation erreicht ist -> OK fr Generation
;TODO Beim Einzeichnen mit der Maus die Umgebung vergrern
;TODO Geladene oder aus Liste ausgewhlte Objekte per Maus positionieren -> OK
;TODO Bereich markieren (Gummiband?) und spiegeln/rotieren und speichern
;TODO Bereich um Mauszeiger vergern (Bildschirmlupe)
;TODO Einzelschritt -> OK
;TODO Ausschneiden,Kopieren,Einfgen fr ausgewhlten Bereich ->OK 
;TODO Verschiedene Regelwerke -> testen
;TODO RLE-Decoder -> testen
;TODO Test, wie schnell ist Conway_Berechnen() ?
;TODO Population/Generation zhlen/anzeigen -> testen fr das Einfgen von Objekten
;TODO Passend zoomen -> testen 
;TODO Bei "Objekt laden" Spielfeldgre anpassen (?)
;TODO Statt "Spielfeld zufllig fllen" Menupunkt "Bereich bzw. Auswahl zufllig fllen" -> OK
;TODO Schrittweite/Scrollbalken im ScrollAreaGadget anpassen
;TODO RLE_Dekodieren: Beim Einfgen in Canvas dekodieren,
;     NICHT in Liste "Koordinaten.sKoordinaten()" -> testen

;FIXME Regel in EigeneObjekte eintragen und anwenden ! -> testen
;FIXME Grenzen beachten beim Einfgen/Kopieren von (geladenen) Objekten in das Spielfeld -> testen
;FIXME "gabrielsp138.rle" prfen, evtl. Fehler in RLE_Dekodieren(), Variablen "m","wert" -> ok


; IDE Options = PureBasic 5.70 LTS (Windows - x64)
; Folding = AAAAAw
; EnableXP
; CompileSourceDirectory